#ifndef SNWNMP_H
#define SNWNMP_H

/* SNWNMP Functions */
extern int snwnmp_process(struct net_device *dev, unsigned char *frame,
			  unsigned len);
extern int snwnmp_handle_trap(struct net_device *dev, unsigned char *frame,
			      unsigned len);
extern int snwnmp_handle_response(struct net_device *dev,
				  unsigned char *frame, unsigned len);
extern int snwnmp_handle_error(struct net_device *dev,
			       unsigned char *frame, unsigned len);
extern int snwnmp_get(struct net_device *dev, unsigned long oid);
extern int snwnmp_set(struct net_device *dev, unsigned long oid,
		      unsigned char *data, unsigned len);
extern void snwnmp_get_roamtable(struct net_device *dev);
extern void snwnmp_get_counters(struct net_device *dev);

/* SNWNMP Ether type */
#define ETH_P_SNWNMP	0x8828

/* SNWNMP header length */
#define SNWNMP_HLEN	12

/* SNWNMP Operations */
#define SNWNMP_GET	0
#define SNWNMP_SET	1
#define SNWNMP_RESPONSE	2
#define SNWNMP_ERROR	3
#define SNWNMP_TRAP	4

struct snwnmp_header {
	unsigned char h_dest[ETH_ALEN];
	unsigned char h_source[ETH_ALEN];
	unsigned short h_proto;
	unsigned char version;
	unsigned char operation;
	unsigned long oid __attribute__ ((packed));
	unsigned char index;
	unsigned char flags;
	unsigned long length __attribute__ ((packed));
};

/* SNWNMP Encryption */
#define MAX_KEY_SIZE		14
#define LARGE_KEY_SIZE		13
#define SMALL_KEY_SIZE		5
#define MAX_KEYS		4


/* OID defines */
#define OID_DOT11_DOT11STATIONID                 	0xFF010100	/* W */
#define OID_DOT11_DOT11MEDIUMOCCUPANCYLIMIT  		0xFF010101	/* W */
#define OID_DOT11_DOT11CFPOLLABLE                	0xFF010102
#define OID_DOT11_DOT11CFPPERIOD            		0xFF010103
#define OID_DOT11_DOT11CFPMAXDURATION       		0xFF010104
#define OID_DOT11_DOT11AUTHENTICATIONRESPONSETIMEOUT	0xFF010105	/* W */
#define OID_DOT11_DOT11PRIVACYOPTIONIMPLEMENTED  	0xFF010106
#define OID_DOT11_DOT11POWERMANAGEMENTMODE  		0xFF010107	/* W */
/* aanvragen ESSID */
#define OID_DOT11_DOT11DESIREDSSID               	0xFF010108	/* W */
#define OID_DOT11_DOT11DESIREDBSSTYPE       		0xFF010109	/* W */
#define OID_DOT11_DOT11OPERATIONALRATESET        	0xFF01010A	/* W */
#define OID_DOT11_DOT11BEACONPERIOD              	0xFF01010B	/* W */
#define OID_DOT11_DOT11DTIMPERIOD                	0xFF01010C	/* W */
#define OID_DOT11_DOT11ASSOCIATIONRESPONSETIMEOUT 	0xFF01010D	/* W */
#define OID_DOT11_DOT11DISASSOCIATEREASON        	0xFF01010E
#define OID_DOT11_DOT11DISASSOCIATESTATION  		0xFF01010F
#define OID_DOT11_DOT11DEAUTHENTICATEREASON  		0xFF010110
#define OID_DOT11_DOT11DEAUTHENTICATESTATION  		0xFF010111
#define OID_DOT11_DOT11AUTHENTICATEFAILSTATUS    	0xFF010112
#define OID_DOT11_DOT11AUTHENTICATEFAILSTATION   	0xFF010113

#define OID_DOT11_DOT11AUTHENTICATIONALGORITHM   	0xFF010114
#define OID_DOT11_DOT11AUTHENTICATIONALGORITHMENABLE	0xFF010115	/* W */

#define OID_DOT11_DOT11WEPDEFAULTKEY        		0xFF010116	/* W */

#define OID_DOT11_DOT11WEPKEYMAPPINGADDRESS  		0xFF010117	/* W */
#define OID_DOT11_DOT11WEPKEYMAPPINGKEY     		0xFF010118	/* W */
#define OID_DOT11_DOT11WEPKEYMAPPINGWEPON        	0xFF010119	/* W */

#define OID_DOT11_DOT11PRIVACYINVOKED       		0xFF01011A	/* W */
#define OID_DOT11_DOT11WEPDEFAULTKEYID           	0xFF01011B	/* W */
#define OID_DOT11_DOT11WEPKEYMAPPINGLENGTH  		0xFF01011C	/* W */
#define OID_DOT11_DOT11EXCLUDEUNENCRYPTED        	0xFF01011D	/* W */
#define OID_DOT11_DOT11WEPICVERRORCOUNT     		0xFF01011E
#define OID_DOT11_DOT11WEPEXCLUDEDCOUNT     		0xFF01011F

#define OID_DOT11_DOT11MACADDRESS                	0xFF010120
#define OID_DOT11_DOT11RTSTHRESHOLD              	0xFF010121	/* W */
#define OID_DOT11_DOT11SHORTRETRYLIMIT           	0xFF010122	/* W */
#define OID_DOT11_DOT11LONGRETRYLIMIT       		0xFF010123	/* W */
#define OID_DOT11_DOT11FRAGMENTATIONTHRESHOLD    	0xFF010124	/* W */
#define OID_DOT11_DOT11MAXTRANSMITLIFETIME   		0xFF010125	/* W */
#define OID_DOT11_DOT11MAXRECEIVELIFETIME        	0xFF010126	/* W */
#define OID_DOT11_DOT11MANUFACTURERID       		0xFF010127
#define OID_DOT11_DOT11PRODUCTID            		0xFF010128

#define OID_DOT11_DOT11TRANSMITTEDFRAGMENTCOUNT  	0xFF010129
#define OID_DOT11_DOT11MULTICASTTRANSMITTEDFRAMECOUNT	0xFF01012A
#define OID_DOT11_DOT11FAILEDCOUNT               	0xFF01012B
#define OID_DOT11_DOT11RETRYCOUNT                	0xFF01012C
#define OID_DOT11_DOT11MULTIPLERETRYCOUNT        	0xFF01012D
#define OID_DOT11_DOT11FRAMEDUPLICATECOUNT  		0xFF01012E
#define OID_DOT11_DOT11RTSSUCCESSCOUNT      		0xFF01012F
#define OID_DOT11_DOT11RTSFAILURECOUNT           	0xFF010130
#define OID_DOT11_DOT11ACKFAILURECOUNT           	0xFF010131
#define OID_DOT11_DOT11RECEIVEDFRAGMENTCOUNT  		0xFF010132
#define OID_DOT11_DOT11MULTICASTRECEIVEDFRAMECOUNT 	0xFF010133
#define OID_DOT11_DOT11FCSERRORCOUNT        		0xFF010134
#define OID_DOT11_DOT11TRANSMITTEDFRAMECOUNT  		0xFF010135
#define OID_DOT11_DOT11WEPUNDECRYPTABLECOUNT  		0xFF010136

#define OID_DOT11_DOT11GROUPADDRESSES       		0xFF010137	/* W */

#define OID_DOT11_DOT11MANUFACTUREROUI           	0xFF010138
#define OID_DOT11_DOT11MANUFACTURERNAME     		0xFF010139
#define OID_DOT11_DOT11MANUFACTURERPRODUCTNAME  	0xFF01013A
#define OID_DOT11_DOT11MANUFACTURERPRODUCTVERSION 	0xFF01013B

#define OID_DOT11_DOT11PHYTYPE                   	0xFF01013C
#define OID_DOT11_DOT11CURRENTREGDOMAIN     		0xFF01013D	/* W */
#define OID_DOT11_DOT11TEMPTYPE                  	0xFF01013E
#define OID_DOT11_DOT11CURRENTTXANTENNA     		0xFF01013F	/* W */
#define OID_DOT11_DOT11DIVERSITYSUPPORT          	0xFF010140
#define OID_DOT11_DOT11CURRENTRXANTENNA     		0xFF010141
/* W */
#define OID_DOT11_DOT11NUMBERSUPPORTEDPOWERLEVELS 	0xFF010142
#define OID_DOT11_DOT11TXPOWERLEVEL1        		0xFF010143
#define OID_DOT11_DOT11TXPOWERLEVEL2        		0xFF010144
#define OID_DOT11_DOT11TXPOWERLEVEL3        		0xFF010145
#define OID_DOT11_DOT11TXPOWERLEVEL4        		0xFF010146
#define OID_DOT11_DOT11TXPOWERLEVEL5        		0xFF010147
#define OID_DOT11_DOT11TXPOWERLEVEL6        		0xFF010148
#define OID_DOT11_DOT11TXPOWERLEVEL7        		0xFF010149
#define OID_DOT11_DOT11TXPOWERLEVEL8        		0xFF01014A
#define OID_DOT11_DOT11CURRENTTXPOWERLEVEL  		0xFF01014B	/* W */
#define OID_DOT11_DOT11CURRENTCHANNEL       		0xFF01014C	/* W */
#define OID_DOT11_DOT11CCAMODESUPPORTED     		0xFF01014D
#define OID_DOT11_DOT11CURRENTCCAMODE       		0xFF01014E	/* W */
#define OID_DOT11_DOT11EDTHRESHOLD               	0xFF01014F	/* W */
#define OID_DOT11_DOT11REGDOMAINSSUPPORTED  		0xFF010150
#define OID_DOT11_DOT11SUPPORTEDTXANTENNAS  		0xFF010151	/* W */
#define OID_DOT11_DOT11SUPPORTEDRXANTENNAS  		0xFF010152	/* W */
#define OID_DOT11_DOT11DIVERSITYSELECTIONRX      	0xFF010153	/* W */
#define OID_DOT11_DOT11SUPPORTEDDATARATESTX  		0xFF010154
#define OID_DOT11_DOT11SUPPORTEDDATARATESRX  		0xFF010155

#define OID_NWN_SMTASSOCIATIONID            		0xFF0101A0

#define OID_NWN_SMTCAPABILITYINFO                	0xFF0101A1
#define OID_NWN_SMTPOWERSAVEINTERVAL        		0xFF0101A2	/* W */
#define OID_NWN_SMTLISTENINTERVAL                	0xFF0101A3	/* W */
#define OID_NWN_SMTATIMWINDOW                    	0xFF0101A4	/* W */
/* */
#define OID_NWN_SMTOPERATIONALCHANNELS      		0xFF0101A5	/* W */
/* mac adres accespoint */
#define OID_NWN_SMTCURRENTBSSID             		0xFF0101A6

/* ESSID */
#define OID_NWN_SMTCURRENTSSID                   	0xFF0101A7
#define OID_NWN_SMTCURRENTBSSTYPE                	0xFF0101A8
#define OID_NWN_SMTPUBLICKEYENABLE               	0xFF0101A9
	 /* Notice non-sequential numbering */
#define OID_NWN_SMTQUALITYLEVEL0            		0xFF0101D0

#define OID_NWN_SMTQUALITYLEVEL1            		0xFF0101D1
#define OID_NWN_SMTQUALITYLEVEL2            		0xFF0101D2
#define OID_NWN_SMTQUALITYPENALTY                	0xFF0101D3
#define OID_NWN_SMTSTATIONDBTIMEOUT              	0xFF0101D4
#define OID_NWN_SMTQUALITYINDICATOR			0xFF0101D5
#define OID_NWN_SMTQUALITYUPPERLIMIT			0xFF0101D6
#define OID_NWN_SMTQUALITYLOWERLIMIT			0xFF0101D7

#define OID_NWN_ROAMSCANTYPE                     	0xFF0101AA	/* W */
#define OID_NWN_ROAMSCANINTERVAL            		0xFF0101AB	/* W */
#define OID_NWN_ROAMPROBEDELAY              		0xFF0101AC	/* W */
#define OID_NWN_ROAMMINCHANNELTIME          		0xFF0101AD	/* W */
#define OID_NWN_ROAMMAXCHANNELTIME          		0xFF0101AE	/* W */
#define OID_NWN_ROAMJOINTIMEOUT             		0xFF0101AF	/* W */
#define OID_NWN_ROAMBEACONPERIODTIMEOUT     		0xFF0101B0	/* W */
#define OID_NWN_ROAMDONTSWITCH              		0xFF0101B1	/* W */
#define OID_NWN_ROAMBLACKOUT                     	0xFF0101B2	/* W */
#define OID_NWN_ROAMDISASSOCIATETIME        		0xFF0101B3	/* W */
#define OID_NWN_ROAMHANDOFFTIME             		0xFF0101B4	/* W */
#define OID_NWN_ROAMWEIGHTMETRIC1                	0xFF0101B5	/* W */
#define OID_NWN_ROAMWEIGHTMETRIC2                	0xFF0101B6	/* W */
#define OID_NWN_ROAMWEIGHTMETRIC3                	0xFF0101B7	/* W */
#define OID_NWN_ROAMWEIGHTMETRIC4                	0xFF0101B8	/* W */
#define OID_NWN_ROAMWEIGHTMETRIC5                	0xFF0101B9	/* W */
#define OID_NWN_ROAMWEIGHTMETRIC6                	0xFF0101BA	/* W */
#define OID_NWN_ROAMWEIGHTMETRIC7                	0xFF0101BB	/* W */
#define OID_NWN_ROAMWEIGHTMETRIC8                	0xFF0101BC	/* W */
#define OID_NWN_ROAMMISC1                        	0xFF0101BD	/* W */
#define OID_NWN_ROAMRSSIAVERAGETIME1        		0xFF0101BD	/* W */
   /* Note: same as previous entry, old name */
#define OID_NWN_ROAMMISC2                        	0xFF0101BE	/* W */
#define OID_NWN_ROAMRSSIAVERAGETIME2        		0xFF0101BE	/* W */
   /* Note: same as previous entry, old name */
#define OID_NWN_ROAMTABLELENGTH             		0xFF0101BF

#define OID_NWN_ROAMBSSID                        	0xFF0101C0
#define OID_NWN_ROAMSSID                    		0xFF0101C1
#define OID_NWN_ROAMBSSTYPE                 		0xFF0101C2
#define OID_NWN_ROAMCHANNEL                 		0xFF0101C3
#define OID_NWN_ROAMAGE                          	0xFF0101C4
#define OID_NWN_ROAMQUALITY                 		0xFF0101C5
#define OID_NWN_ROAMLOAD                    		0xFF0101C6
#define OID_NWN_ROAMBEACONPERIOD            		0xFF0101C7
#define OID_NWN_ROAMDTIMPERIOD                   	0xFF0101C8
#define OID_NWN_ROAMCAPABILITYINFORMATION        	0xFF0101C9
#define OID_NWN_ROAMRATES                        	0xFF0101CC
#define OID_NWN_ROAMRSSI				0xFF0101CD

#define OID_NWN_BSSCHANGETRAP				0xFF0101D5

#define OID_NWN_DRIVER_VERSION                   	0xFF0101E0
#define OID_NWN_UPGRADE                          	0xFF0101E1
#define OID_NWN_HARDCONFIG                       	0xFF0101E2

#define OID_NWN_TESTREGISTER1                    	0xFF0101F0
#define OID_NWN_TESTREGISTER2                    	0xFF0101F1
#define OID_NWN_TESTREGISTER3                    	0xFF0101F2
#define OID_NWN_TESTREGISTER4                    	0xFF0101F3
#define OID_NWN_TESTREGISTER5                    	0xFF0101F4
#define OID_NWN_TESTREGISTER6                    	0xFF0101F5
#define OID_NWN_TESTREGISTER7                    	0xFF0101F6
#define OID_NWN_TESTREGISTER8                    	0xFF0101F7
#define OID_NWN_TESTREGISTER9                    	0xFF0101F8
#define OID_NWN_TESTREGISTER10                   	0xFF0101F9
#define OID_NWN_TESTREGISTER11                   	0xFF0101FA
#define OID_NWN_TESTREGISTER12                   	0xFF0101FB
#define OID_NWN_TESTREGISTER13                   	0xFF0101FC
#define OID_NWN_TESTREGISTER14                   	0xFF0101FD
#define OID_NWN_TESTREGISTER15                   	0xFF0101FE
#define OID_NWN_TESTREGISTER16                   	0xFF0101FF

/* 3Com OID's */
#define OID_3COM_ADHOC					0xFF010210
#define OID_3COM_INFRASTRUCTURE				0xFF010211
#define OID_3COM_ACTIVE_PROBING				0xFF010309

#define OID_3COM_RESETTRAP				0xFF0103FF

#endif				/* SNWNMP_H */
