/*
  Copyright 2004, 2005 Jean-Baptiste Note

  This file is part of prism54usb.

  prism54usb is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  prism54usb is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with prism54usb; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/module.h>

#include "islsm_log.h"

#if ISL_DEBUG
unsigned int     isl_debug = 0;
module_param_named(debug, isl_debug, uint, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(debug, "Adjust verbosity of the driver.");
EXPORT_SYMBOL(isl_debug);

void isl_dump_bytes(const void *_data, int num)
{
	const u8 *data = (const u8 *) _data;
	const u8 *ptr = data;

	if (num <= 0) {
		printk("\n");
		return;
	}

	while (num >= 16) {
		printk(KERN_CRIT "%04x:"
		       "%02X %02X %02X %02X %02X %02X %02X %02X "
		       "%02X %02X %02X %02X %02X %02X %02X %02X\n",
		       (uint16_t) (ptr - data),
		       ptr[0], ptr[1], ptr[2], ptr[3],
		       ptr[4], ptr[5], ptr[6], ptr[7],
		       ptr[8], ptr[9], ptr[10], ptr[11],
		       ptr[12], ptr[13], ptr[14], ptr[15]);
		num -= 16;
		ptr += 16;
	}
	if (num > 0) {
		printk(KERN_CRIT "%04x:", (uint16_t) (ptr - data));
		while (--num > 0)
			printk("%02X ", *ptr++);
		printk("%02X\n", *ptr);
	}
}

EXPORT_SYMBOL(isl_dump_bytes);

#endif /* ISL_DEBUG */

#if ISL_DEBUG > 1

static int isl_debug_func_indent;

#define DEBUG_TSC 0
#define FUNC_INDENT_INCREMENT 2

#if DEBUG_TSC
#define TIMESTAMP(d) unsigned long d; rdtscl(d)
#else
#define TIMESTAMP(d) unsigned long d = jiffies
#endif

static const char
 spaces[] = "          " "          ";	/* Nx10 spaces */

void
isl_fn_enter(const char *funcname)
{
	int indent;
	TIMESTAMP(d);

	indent = isl_debug_func_indent;
	if (indent >= sizeof (spaces))
		indent = sizeof (spaces) - 1;

	printk("%08ld %s==> %s\n",
	       d % 100000000,
	       spaces + (sizeof (spaces) - 1) - indent, funcname);

	isl_debug_func_indent += FUNC_INDENT_INCREMENT;
}

EXPORT_SYMBOL(isl_fn_enter);

void
isl_fn_exit(const char *funcname)
{
	int indent;
	TIMESTAMP(d);

	isl_debug_func_indent -= FUNC_INDENT_INCREMENT;

	indent = isl_debug_func_indent;
	if (indent >= sizeof (spaces))
		indent = sizeof (spaces) - 1;

	printk("%08ld %s<== %s\n",
	       d % 100000000,
	       spaces + (sizeof (spaces) - 1) - indent, funcname);
}

EXPORT_SYMBOL(isl_fn_exit);

void
isl_fn_exit_v(const char *funcname, int v)
{
	int indent;
	TIMESTAMP(d);

	isl_debug_func_indent -= FUNC_INDENT_INCREMENT;

	indent = isl_debug_func_indent;
	if (indent >= sizeof (spaces))
		indent = sizeof (spaces) - 1;

	printk("%08ld %s<== %s: %08X\n",
	       d % 100000000,
	       spaces + (sizeof (spaces) - 1) - indent, funcname, v);
}

EXPORT_SYMBOL(isl_fn_exit_v);

#endif				/* ISL_DEBUG > 1 */
